@@ -49,7 +49,7 @@ pub fn register_shutdown_pipe_handler(manager: &Socket) {
         .expect("failed to set up new pipe handler");
 }
 
-pub async fn worker_main(goose_attack: &GooseAttack) -> GooseAttack {
+pub(crate) async fn worker_main(goose_attack: &GooseAttack) -> GooseAttack {
     // Creates a TCP address.
     let address = format!(
         "tcp://{}:{}",
@@ -219,19 +219,29 @@ pub async fn worker_main(goose_attack: &GooseAttack) -> GooseAttack {
     // Use the run_time from the Manager so Worker can shut down in a timely manner.
     worker_goose_attack.run_time = run_time;
     worker_goose_attack.weighted_users = weighted_users;
+    // This is a Worker instance, not a Manager instance.
+    worker_goose_attack.configuration.manager = false;
     worker_goose_attack.configuration.worker = true;
-    // The requests_file option is configured on the Worker.
-    worker_goose_attack.configuration.requests_file =
-        goose_attack.configuration.requests_file.to_string();
-    // The requests_format option is configured on the Worker.
-    worker_goose_attack.configuration.requests_format =
-        goose_attack.configuration.requests_format.to_string();
-    // The debug_file option is configured on the Worker.
-    worker_goose_attack.configuration.debug_file =
-        goose_attack.configuration.debug_file.to_string();
+    // The request_log option is configured on the Worker.
+    worker_goose_attack.configuration.request_log =
+        goose_attack.configuration.request_log.to_string();
+    // The request_format option is configured on the Worker.
+    worker_goose_attack.configuration.request_format =
+        goose_attack.configuration.request_format.clone();
+    // The task_log option is configured on the Worker.
+    worker_goose_attack.configuration.task_log = goose_attack.configuration.task_log.to_string();
+    // The task_format option is configured on the Worker.
+    worker_goose_attack.configuration.task_format = goose_attack.configuration.task_format.clone();
+    // The error_log option is configured on the Worker.
+    worker_goose_attack.configuration.error_log = goose_attack.configuration.error_log.to_string();
+    // The error_format option is configured on the Worker.
+    worker_goose_attack.configuration.error_format =
+        goose_attack.configuration.error_format.clone();
+    // The debug_log option is configured on the Worker.
+    worker_goose_attack.configuration.debug_log = goose_attack.configuration.debug_log.to_string();
     // The debug_format option is configured on the Worker.
     worker_goose_attack.configuration.debug_format =
-        goose_attack.configuration.debug_format.to_string();
+        goose_attack.configuration.debug_format.clone();
     // The throttle_requests option is set on the Worker.
     worker_goose_attack.configuration.throttle_requests =
         goose_attack.configuration.throttle_requests;
