@@ -5,11 +5,11 @@ use std::net::TcpStream;
 use std::{str, thread, time};
 use tungstenite::Message;
 
+use goose::config::GooseConfiguration;
 use goose::controller::{
     GooseControllerCommand, GooseControllerWebSocketRequest, GooseControllerWebSocketResponse,
 };
 use goose::prelude::*;
-use goose::GooseConfiguration;
 
 mod common;
 
@@ -47,11 +47,14 @@ struct TestState {
     // A TCP socket if testing the telnet Controller.
     telnet_stream: Option<TcpStream>,
     // A TCP socket if testing the WebSocket Controller.
+    #[cfg(not(feature = "rustls-tls"))]
+    websocket_stream: Option<tungstenite::WebSocket<std::net::TcpStream>>,
+    #[cfg(feature = "rustls-tls")]
     websocket_stream: Option<
         tungstenite::WebSocket<
             tungstenite::stream::Stream<
                 std::net::TcpStream,
-                native_tls::TlsStream<std::net::TcpStream>,
+                rustls::StreamOwned<rustls::ClientSession, TcpStream>,
             >,
         >,
     >,
